const { plugin: { pluginObjectTypes, pluginScope } } = require('@kl/constants')
const path = require('path')
const fs = require('fs')
const consts = require('./consts')
const lo = require('lodash')

function PluginServerAPI (kscApi) {
  const policy = require('./converters/policy/index')()
  const group_tasks = require('./converters/tasks/index')()
  const distributive = require('./converters/distributive/index')()
  const settings = require('./converters/settings/index')()
  const local_tasks = require('./converters/local_tasks/index')()

  async function getFormModel ({ context: { object: { id, type, parentId }, scope }, form, type: taskType }) {
        console.log('ksws_windows:> getFormModel ' + JSON.stringify(arguments))
        var data = { }
        switch (type) {
            case pluginObjectTypes.policy:
                data = await policy.getPolicyData(kscApi, id, form)
                break
            case pluginObjectTypes.task:
                data = await group_tasks.getTaskData ({kscApi, taskId: id, hostId: false, taskType, formType: form})
                break
            case pluginObjectTypes.taskInitial:
                let defData = await getTaskDefaults({taskType, scope})
                data = await group_tasks.getWizardTaskData({kscApi, taskType, formType: form, defData})
                break
            case pluginObjectTypes.localTask:
                data = await local_tasks.getTaskData(kscApi, id, parentId, taskType, form);
                break
            case pluginObjectTypes.distributive:
                data = await distributive.getPackageData ({kscApi, packageId: id, form})
                break
            case pluginObjectTypes.settings:
                data = await settings.getSettingsData(kscApi, form, parentId);
                break
        }
        return data
  }

  async function setFormModel ({ context: { scope, object: { type, parentId } }, form, taskType }, data) {
      console.log('ksws_windows:> setFormModel ' + JSON.stringify(arguments))
      switch (type) {
          
          case pluginObjectTypes.policy:
              if(form !== undefined)
                await policy.updatePolicyData(kscApi, form, data)
              break
          case pluginObjectTypes.task:
              if (scope === pluginScope.editObject)
                  await group_tasks.updateTaskData ({kscApi, modelData: data, taskType, formType: form})
              break
          case pluginObjectTypes.taskInitial:
              await group_tasks.updateTaskData ({kscApi, modelData: data, taskType, formType: form})
              break
          case pluginObjectTypes.distributive:
              data = await distributive.savePackageData ({kscApi, modelData: data, form})
              break
          case pluginObjectTypes.settings:
              settings.updateSettingsData(kscApi, form, data);
              break;
          case pluginObjectTypes.localTask:
              if (scope === pluginScope.editObject)
                  await local_tasks.updateTaskData ({kscApi, modelData: data, taskType, formType: form})
              break
      }
      return true
  }

  async function beforeApplyChanges ({ object: { id, type }, scope }) {
      console.log('ksws_windows:> beforeApplyChanges ' + JSON.stringify(arguments))
      switch (type) {
          case pluginObjectTypes.policy:
              policy.commitPolicyData(kscApi, scope)
              break
          case pluginObjectTypes.settings:
              settings.commitSettingsData(kscApi)
              break
      }
  }

  async function applyChanges ({ object: { id, type, parentId }, scope }) {
      console.log('ksws_windows:> applyChanges ' + JSON.stringify(arguments))
      switch (type) {
          case pluginObjectTypes.policy:
              policy.savePolicyData(kscApi, id, scope)
              break
          case pluginObjectTypes.settings:
              settings.saveSettingsData(kscApi, id, parentId)
              break
      }
  }

  /**
   * Метод для получения данных от плагина по имени файла
   * @param fileName - имя файла с запрашиваемыми данными
   * @returns {Promise<string | Buffer>} - текстовое содержимое файла
   */
  async function getAssetByFileName ({fileName}) {
    try {
      const parsedFileName = path.parse(fileName)
      const filePath = path.join(__dirname, 'assets', parsedFileName.base)
      const fileContent = fs.readFileSync(filePath, 'utf-8')
      return fileContent
    } catch (error) {
      console.error(error)
      throw new Error('File not found')
    }
  }

  /**
   * Запуск продукта на устройстве
   * @param {string} hostName - уникальное имя хоста
   * @return {Promise.<*>}
   */
  async function startProductOnHost ({ hostName }) {
    return hostName
  }

  /**
   * Остановка продукта на устройстве
   * @param {string} hostName - уникальное имя хоста
   * @return {Promise.<*>}
   */
  async function stopProductOnHost ({ hostName }) {
    return hostName
  }

  /**
   * Возвращает параметры задачи по-умолчанию
   * @param taskType - тип задачи
   * @param scope - контекст выполнения операции
   * @returns {Promise.<*>}
   */
  async function getTaskDefaults({ taskType, scope }) {
      console.log('ksws_windows:> getTaskDefaults ' + JSON.stringify(arguments))

      let task = {}
      switch (scope) {
          case pluginScope.newObject:
              const tasksRaw = await getAssetByFileName({ fileName: 'task-defaults.json' })
              const tasks = JSON.parse(tasksRaw)
              task = tasks[taskType]
              break
          case pluginScope.newQSWObject:
          case pluginScope.newCloudProtectionObject:
/*
              const wizardsDefaultsRaw = await getAssetByFileName({ fileName: 'wizards-defaults.json' })
              const sectionName = (scope === pluginScope.newQSWObject) ? 'tasksQsw' : 'tasksPCloud'
              const qswTasks = JSON.parse(wizardsDefaultsRaw)[sectionName]
              task = qswTasks['common']
              let taskData = _.find(qswTasks['taskData'], { TASK_NAME: taskType })
              if (taskData !== undefined) {
                  _.merge(task, taskData)
              }
*/
              break

      }

      return task
  }

  /**
   * Возвращает продуктовые настройки задачи
   * @param taskData - настройки задачи KSC
   * @param scope - контекст выполнения операции
   * @returns {Promise.<*>}
   */
  async function getTaskAdditionalParams ({ task, scope }) {
    console.log('ksws_windows:> getTaskAdditionalParams ' + JSON.stringify(arguments))
      switch (scope) {
          case pluginScope.newObject:
              return group_tasks.getTaskAdditionalParams()
              break
          case pluginScope.editObject:
              if(task.isLocalTask)
                  return local_tasks.getTaskAdditionalParams(task.TASK_NAME)
              else
                return group_tasks.getTaskAdditionalParams()
              break
          case pluginScope.newQSWObject:
              return task.TASK_ADDITIONAL_PARAMS
              break

      }
   }

  /**
   * Return task registration data
   * @param taskType - required task type
   * @returns {Promise.<object>}
   */
  async function getTaskRegistrationData ({ taskType }) {
    const taskTypesJson = await getAssetByFileName({ fileName: 'task-types.json' })
    const taskTypes = JSON.parse(taskTypesJson)
    const taskData = lo.find(taskTypes, { type: taskType })
    if (!taskData) {
      throw new Error('Unknown task type')
    }
    return taskData
  }

  async function getEula({ type, localization = 'en-us' }) {
    const ksnPath = path.resolve(__dirname, `./assets/ksn/${localization}.txt`)
    return fs.readFileSync(ksnPath, 'utf8')
  }

  return {
    getFormModel,
    setFormModel,
    beforeApplyChanges,
    applyChanges,
    getAssetByFileName,
    startProductOnHost,
    stopProductOnHost,
    getTaskAdditionalParams,
    getTaskDefaults,
    getTaskRegistrationData,
    getEula
  }
}

module.exports = PluginServerAPI